@@ -22,16 +22,17 @@
 
 import ctypes
 import ctypes.util
+import datetime
 import functools
+import inspect
+import multiprocessing
+import os
+import platform
+import sys
 import threading
 import traceback
-import inspect
-import datetime
-import platform
-import os
-import sys
-import multiprocessing
 
+import fdb
 from fdb import six
 
 _network_thread = None
@@ -203,7 +204,9 @@ def transactional(*tr_args, **tr_kwargs):
 
     It is important to note that the wrapped method may be called
     multiple times in the event of a commit failure, until the commit
-    succeeds.
+    succeeds.  This restriction requires that the wrapped function
+    may not be a generator, or a function that returns a closure that
+    contains the `tr` object.
 
     If given a Transaction, the Transaction will be passed into the
     wrapped code, and WILL NOT be committed at completion of the
@@ -247,9 +250,14 @@ def transactional(*tr_args, **tr_kwargs):
                     except FDBError as e:
                         yield asyncio.From(tr.on_error(e.code))
         else:
-
             @functools.wraps(func)
             def wrapper(*args, **kwargs):
+                # We can't throw this from the decorator, as when a user runs
+                # >>> import fdb ; fdb.api_version(630)
+                # the code above uses @transactional before the API version is set
+                if fdb.get_api_version() >= 630 and inspect.isgeneratorfunction(func):
+                    raise ValueError("Generators can not be wrapped with fdb.transactional")
+
                 if isinstance(args[index], TransactionRead):
                     return func(*args, **kwargs)
 
@@ -262,8 +270,11 @@ def transactional(*tr_args, **tr_kwargs):
                 # last = start
 
                 while not committed:
+                    ret = None
                     try:
                         ret = func(*largs, **kwargs)
+                        if fdb.get_api_version() >= 630 and inspect.isgenerator(ret):
+                            raise ValueError("Generators can not be wrapped with fdb.transactional")
                         tr.commit().wait()
                         committed = True
                     except FDBError as e:
@@ -405,7 +416,7 @@ class TransactionRead(_FDBBase):
 
     def get_read_version(self):
         """Get the read version of the transaction."""
-        return FutureVersion(self.capi.fdb_transaction_get_read_version(self.tpointer))
+        return FutureInt64(self.capi.fdb_transaction_get_read_version(self.tpointer))
 
     def get(self, key):
         key = keyToBytes(key)
@@ -449,6 +460,17 @@ class TransactionRead(_FDBBase):
         if isinstance(key, slice):
             return self.get_range(key.start, key.stop, reverse=(key.step == -1))
         return self.get(key)
+    
+    def get_estimated_range_size_bytes(self, begin_key, end_key):
+        if begin_key is None:
+            begin_key = b''
+        if end_key is None:
+            end_key = b'\xff'
+        return FutureInt64(self.capi.fdb_transaction_get_estimated_range_size_bytes(
+            self.tpointer,
+            begin_key, len(begin_key),
+            end_key, len(end_key)
+            ))
 
 
 class Transaction(TransactionRead):
@@ -541,6 +563,10 @@ class Transaction(TransactionRead):
         self.capi.fdb_transaction_get_committed_version(self.tpointer, ctypes.byref(version))
         return version.value
 
+    def get_approximate_size(self):
+        """Get the approximate commit size of the transaction."""
+        return FutureInt64(self.capi.fdb_transaction_get_approximate_size(self.tpointer))
+
     def get_versionstamp(self):
         return Key(self.capi.fdb_transaction_get_versionstamp(self.tpointer))
 
@@ -687,12 +713,12 @@ class FutureVoid(Future):
         return None
 
 
-class FutureVersion(Future):
+class FutureInt64(Future):
     def wait(self):
         self.block_until_ready()
-        version = ctypes.c_int64()
-        self.capi.fdb_future_get_version(self.fpointer, ctypes.byref(version))
-        return version.value
+        value = ctypes.c_int64()
+        self.capi.fdb_future_get_int64(self.fpointer, ctypes.byref(value))
+        return value.value
 
 
 class FutureKeyValueArray(Future):
@@ -1216,6 +1242,8 @@ if platform.system() == 'Windows':
     capi_name = 'fdb_c.dll'
 elif platform.system() == 'Linux':
     capi_name = 'libfdb_c.so'
+elif platform.system() == 'FreeBSD':
+    capi_name = 'libfdb_c.so'
 elif platform.system() == 'Darwin':
     capi_name = 'libfdb_c.dylib'
 elif sys.platform == 'win32':
@@ -1359,9 +1387,9 @@ def init_c_api():
     _capi.fdb_future_get_error.restype = int
     _capi.fdb_future_get_error.errcheck = check_error_code
 
-    _capi.fdb_future_get_version.argtypes = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_int64)]
-    _capi.fdb_future_get_version.restype = ctypes.c_int
-    _capi.fdb_future_get_version.errcheck = check_error_code
+    _capi.fdb_future_get_int64.argtypes = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_int64)]
+    _capi.fdb_future_get_int64.restype = ctypes.c_int
+    _capi.fdb_future_get_int64.errcheck = check_error_code
 
     _capi.fdb_future_get_key.argtypes = [ctypes.c_void_p, ctypes.POINTER(ctypes.POINTER(ctypes.c_byte)),
                                          ctypes.POINTER(ctypes.c_int)]
@@ -1420,6 +1448,9 @@ def init_c_api():
                                                 ctypes.c_int, ctypes.c_int]
     _capi.fdb_transaction_get_range.restype = ctypes.c_void_p
 
+    _capi.fdb_transaction_get_estimated_range_size_bytes.argtypes = [ctypes.c_void_p, ctypes.c_void_p, ctypes.c_int, ctypes.c_void_p, ctypes.c_int]
+    _capi.fdb_transaction_get_estimated_range_size_bytes.restype = ctypes.c_void_p
+
     _capi.fdb_transaction_add_conflict_range.argtypes = [ctypes.c_void_p, ctypes.c_void_p, ctypes.c_int, ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
     _capi.fdb_transaction_add_conflict_range.restype = ctypes.c_int
     _capi.fdb_transaction_add_conflict_range.errcheck = check_error_code
@@ -1453,6 +1484,9 @@ def init_c_api():
     _capi.fdb_transaction_get_committed_version.restype = ctypes.c_int
     _capi.fdb_transaction_get_committed_version.errcheck = check_error_code
 
+    _capi.fdb_transaction_get_approximate_size.argtypes = [ctypes.c_void_p]
+    _capi.fdb_transaction_get_approximate_size.restype = ctypes.c_void_p
+
     _capi.fdb_transaction_get_versionstamp.argtypes = [ctypes.c_void_p]
     _capi.fdb_transaction_get_versionstamp.restype = ctypes.c_void_p
 
